import logging

from elasticsearch import Elasticsearch
from fastapi import HTTPException

from app.conf.elasticConfig import ELASTIC_SEARCH_SERVICE_URI, ELASTIC_PASSWORD
from app.models import SearchIndex
from app.searchUtils import get_hits_as_result

logger = logging.getLogger(__name__)


class SearchManager:
    def __init__(self):

        logger.info("Initializing Search Manager..")

        # Create the client instance
        self.client = Elasticsearch(
            ELASTIC_SEARCH_SERVICE_URI,
            verify_certs=False,
            basic_auth=("elastic", ELASTIC_PASSWORD)
        )

        if not self.client.ping:
            raise ValueError("Connection failed")

        logger.info("Adding Default Indices to our Elastic Search..")
        for default_index in list(SearchIndex):
            if not self.client.indices.exists(index=default_index):
                self.client.indices.create(index=default_index)
                logger.info("index: " + default_index + " Has Been Added Successfully")
            else:
                logger.info("index: " + default_index + " is duplicated")

        logger.info("Elastic Search Client Connected Successfully")

    def execute_query(self, list_of_indices, query_body, limit, with_grouping):
        json_resp = {}
        try:
            es_resp = self.client.search(index=list_of_indices, body=query_body, size=limit)
            # logger.info("response for search: " + str(es_resp))
        except ConnectionError as err:
            raise HTTPException(status_code=404, detail="Cannot connect to elastic, " + str(err))
        except Exception as err:
            raise HTTPException(status_code=404, detail=err)
        logger.info("For index(or indices): %s, Got %d Hits" % (list_of_indices, es_resp['hits']['total']['value']))
        get_hits_as_result(es_resp, json_resp, with_grouping)
        return json_resp



import logging

import uvicorn
from fastapi import FastAPI
from starlette.responses import RedirectResponse

from app.conf.basicConfig import SERVICE_URI, SERVICE_PORT
from app.routers import searchOperations, indexOperations

tags_metadata = [
    {
        "name": "Manipulation",
        "description": "Operations with Documents. This is a pre-configuration for our search service",
    },
    {
        "name": "Primary Search",
        "description": "The default search API to use",
    },
    {
        "name": "Other Types of Search",
        "description": "Search for Documents with Several Types.",
    },
    {
        "name": "Other Types of Informing",
        "description": "Different Types of Informing Elastic Search APIs."
    }
]

app = FastAPI(openapi_tags=tags_metadata, docs_url="/search-service/docs", openapi_url="/search-service/openapi.json")
app.include_router(searchOperations.primary_router)
app.include_router(indexOperations.primary_router)
# app.include_router(searchOperations.additional_router)
# app.include_router(indexOperations.additional_router)


@app.get("/search-service")
async def get_welcome_message():
    return {"message": "Welcome to our Search Service"}


@app.get("/search-service/openapi.json")
async def redirect_openapi():
    return RedirectResponse(url='/openapi.json')


if __name__ == '__main__':
    uvicorn.run('app.main:app', host=SERVICE_URI, port=SERVICE_PORT, reload=True, log_level=logging.DEBUG)



import logging

from fastapi import HTTPException

from app.models import SearchIndex

logger = logging.getLogger(__name__)


def validate_indices(indices: list, all_option=True):
    indices_set = set()
    for index in indices:
        if index.lower() in ["all", "*", "library.*"]:
            if not all_option:
                raise HTTPException(status_code=404, detail="Wrong Index: " + index)
            return ["library.*"]
        if index not in list(SearchIndex):
            raise HTTPException(status_code=404, detail="Wrong Index: " + index)

        indices_set.add(index)
    return list(indices_set)


def get_hits_as_result(es_resp, json_resp, with_grouping=True):
    if with_grouping:
        for hit in es_resp['hits']['hits']:
            if hit['_index'] not in json_resp.keys():
                json_resp[hit['_index']] = {"count": 1, "items": [hit['_source']]}
            else:
                json_resp[hit['_index']]['items'].append(hit['_source'])
                json_resp[hit['_index']]['count'] += 1
    else:
        if json_resp == {}:
            json_resp["all"] = {"count": 0, "items": []}
        for hit in es_resp['hits']['hits']:
            json_resp["all"]["count"] += 1
            json_resp["all"]["items"].append(hit['_source'])
            # json_resp["all"]["items"].append({"score": hit['_score'], "hit": hit['_source']})



import json
import logging
from typing import Union, List, Dict

import elasticsearch
from elasticsearch import helpers
from fastapi import HTTPException, APIRouter

from app.conf.basicConfig import search_manager
from app.conf.logConfig import LoggingRoute
from app.models import Folder, Course, Faculty, Major, SearchIndex, IndexList, User
from app.searchUtils import validate_indices

primary_router = APIRouter(tags=["Search Informer"], prefix="/search-service", route_class=LoggingRoute)
additional_router = APIRouter(tags=["Other Types of Informing"], prefix="/search-service", route_class=LoggingRoute)

logger = logging.getLogger(__name__)


@primary_router.post("/folder/op/add")
async def add_search_info(folder: Folder):
    folder.type = SearchIndex.FOLDER
    search_manager.client.index(index=folder.type, id=folder.id, document=json.dumps(folder.__dict__))
    logger.info("folder with ID %s has been added successfully" % folder.id)
    return "Done"


@primary_router.post("/course/op/add")
async def add_search_info(course: Course):
    course.type = SearchIndex.COURSE
    search_manager.client.index(index=course.type, id=course.id, document=json.dumps(course.__dict__))
    logger.info("course with ID %s has been added successfully" % course.id)
    return "Done"


@primary_router.post("/major/op/add")
async def add_search_info(major: Major):
    major.type = SearchIndex.MAJOR
    search_manager.client.index(index=major.type, id=major.id, document=json.dumps(major.__dict__))
    logger.info("major with ID %s has been added successfully" % major.id)
    return "Done"


@primary_router.post("/faculty/op/add")
async def add_search_info(faculty: Faculty):
    faculty.type = SearchIndex.FACULTY
    search_manager.client.index(index=faculty.type, id=faculty.id, document=json.dumps(faculty.__dict__))
    logger.info("faculty with ID %s has been added successfully" % faculty.id)
    return "Done"


@primary_router.post("/user/op/add")
async def add_search_info(user: User):
    user.type = SearchIndex.USER
    search_manager.client.index(index=user.type, id=user.id, document=json.dumps(user.__dict__))
    logger.info("user with ID %s has been added successfully" % user.id)
    return "Done"


@primary_router.delete("/document/op/delete")
async def delete_document(id: str = None, type: str = None):
    if id is None:
        raise HTTPException(status_code=404, detail="Missing required parameter: id")
    if type is None:
        raise HTTPException(status_code=404, detail="Missing required parameter: type")
    try:
        list_of_indices = validate_indices([type], all_option=False)
        for index in list_of_indices:
            logger.info("index: " + index)
            if search_manager.client.exists(index=index, id=id):
                search_manager.client.delete(index=index, id=id)
                logger.info("Document with ID %s in index %s has been deleted successfully" % (id, index))
    except elasticsearch.NotFoundError:
        raise HTTPException(status_code=404, detail="Cannot Find this document!")

    return "Done"


@additional_router.post("/document/op/add")
async def add_document(document: Dict[str, str]):
    index_name = validate_indices([document.get("type")], all_option=False)
    search_manager.client.index(index=index_name, id=document.get("id"), document=document)
    logger.info("Document with ID %s in index %s has been added successfully" % (document.get("id"), index_name))
    return "Done"


@additional_router.post("/documents/op/add")
async def add_documents(document_list: List[Dict[str, str]]):
    document_list = prepare_documents_to_bulk(document_list)
    response = helpers.bulk(search_manager.client, document_list)
    logger.info(f"Data sent to your ElasticSearch with response: {response}")
    return "Done"


@additional_router.delete("/index/op/delete")
async def delete_index(index_name: Union[str, None] = None):
    if not search_manager.client.indices.exists(index=index_name):
        return "Index " + index_name + " Does not exist"
    es_resp = search_manager.client.indices.delete(index=index_name)
    return es_resp


@additional_router.delete("/document/op/deleteWithBody")
async def delete_documents(index_name: IndexList, document_id: Union[str, None] = None):
    list_of_indices = validate_indices(index_name.indices, all_option=False)
    if document_id is None:
        try:
            search_manager.client.delete_by_query(index=list_of_indices, body={"query": {"match_all": {}}})
            logger.info("Documents in index %s has been deleted successfully" % list_of_indices)
        except Exception:
            raise HTTPException(status_code=404, detail="Failed to delete with index: " + str(list_of_indices))
    else:
        try:
            for index in list_of_indices:
                logger.info("index: " + index)
                if search_manager.client.exists(index=index, id=document_id):
                    search_manager.client.delete(index=index, id=document_id)
                    logger.info("Document with ID %s in index %s has been deleted successfully" % (
                        document_id, index))
        except elasticsearch.NotFoundError:
            raise HTTPException(status_code=404, detail="Cannot Find this document!")

    return "Done"


@additional_router.put("/documents/op/load")
async def load_documents():
    def read_data():
        """Yields json_data from json file."""
        # Test Source
        with open("../conf/initialData/full_format_documents.json", "r") as f:
            json_data = json.load(f)
            logger.info(json.dumps(json_data, indent=2))
            return prepare_documents_to_bulk(json_data)

    data = read_data()
    response = helpers.bulk(search_manager.client, data)
    logger.info(f"Data sent to your ElasticSearch with response: {response}")
    return "Done"


def prepare_documents_to_bulk(document_list):
    # Need to validate every type here
    actions = [
        {
            "_index": document['type'],
            "_id": document['id'],
            "_source": document
        } for document in document_list
    ]
    return actions



import logging
from typing import Union, List

from fastapi import APIRouter

from app.conf.basicConfig import search_manager
from app.conf.elasticConfig import DEFAULT_LIMIT
from app.conf.logConfig import LoggingRoute
from app.models import DetailedCriteria, IndexList, Criteria
from app.searchUtils import validate_indices

additional_router = APIRouter(tags=["Other Types of Search"], prefix="/search-service", route_class=LoggingRoute)
primary_router = APIRouter(tags=["Primary Search"], prefix="/search-service", route_class=LoggingRoute)

logger = logging.getLogger(__name__)


@primary_router.post("/document/op/getAll/")
async def search_documents_by_index(index_list: IndexList, limit: Union[int, None] = DEFAULT_LIMIT,
                                    with_grouping: Union[bool, None] = True):
    list_of_indices = validate_indices(index_list.indices)
    logger.info("Searching for all matches in index(/indices): " + str(list_of_indices))
    query_body = {"query": {"match_all": {}}}
    return search_manager.execute_query(list_of_indices, query_body, limit, with_grouping)


@primary_router.post("/document/op/searchForRecord")
async def search_by_specific_criteria(new_criteria: Criteria, limit: Union[int, None] = DEFAULT_LIMIT,
                                      with_grouping: Union[bool, None] = True):
    list_of_indices = validate_indices(new_criteria.index_list)
    conditions = [{"query_string": {"query": new_criteria.query.replace('"', '\\"'), "default_field": "*"}}]

    query_body = {"query": {"bool": {"must": conditions}}}
    logger.info("Searching in index(/indices): " + str(list_of_indices))
    logger.info("query Body: " + str(query_body))
    return search_manager.execute_query(list_of_indices, query_body, limit, with_grouping)


@additional_router.post("/document/op/searchWithExactMatch")
async def search_with_exact_match(criteria: DetailedCriteria, limit: Union[int, None] = DEFAULT_LIMIT,
                                  with_grouping: Union[bool, None] = True):
    list_of_indices = validate_indices(criteria.index_list)
    conditions = []
    for key, value in criteria.exact_match.items():
        conditions.append({"match": {key: value}})
    for key, value in criteria.query.items():
        conditions.append({"query_string": {"query": value, "default_field": key}})

    query_body = {"query": {"bool": {"must": conditions}}}
    logger.info(
        "Searching in the field(s): " + str(criteria.query.keys()) + " in index(/indices): " + str(list_of_indices))
    logger.info("query Body: " + str(query_body))
    return search_manager.execute_query(list_of_indices, query_body, limit, with_grouping)


@additional_router.get("/document/op/match")
async def search_match(index_list: IndexList, limit: Union[int, None] = DEFAULT_LIMIT, field: Union[str, None] = "",
                       query: Union[str, None] = "", operator: str = "or", with_grouping: Union[bool, None] = True):
    """Perform search by relevance for certain field and query."""
    list_of_indices = validate_indices(index_list.indices)
    logger.info(
        "Searching for " + query + " in the field(s): " + field + " in index(/indices): " + str(list_of_indices))
    query_body = {"query": {"match": {field: {"query": query, "operator": operator}}}}
    return search_manager.execute_query(list_of_indices, query_body, limit, with_grouping)


@additional_router.get("/document/op/wildcard")
async def search_wildcard(index_list: IndexList, limit: Union[int, None] = DEFAULT_LIMIT, field: Union[str, None] = "",
                          query: Union[str, None] = "", with_grouping: Union[bool, None] = True):
    list_of_indices = validate_indices(index_list.indices)
    logger.info(
        "Searching for " + query + " in the field(s): " + field + " in index(/indices): " + str(list_of_indices))
    query_body = {"query": {"wildcard": {field: query}}}
    return search_manager.execute_query(list_of_indices, query_body, limit, with_grouping)


@additional_router.post("/document/op/multiMatch")
async def search_multi_match(index_list: IndexList, limit: Union[int, None] = DEFAULT_LIMIT,
                             fields: Union[List[str], None] = None, query: Union[str, None] = "",
                             with_grouping: Union[bool, None] = True):
    """Perform search by relevance for certain field and query."""
    list_of_indices = validate_indices(index_list.indices)
    logger.info(
        "Searching for " + query + " in the field(s): " + str(fields) + " in index(/indices): " + str(list_of_indices))
    query_body = {"query": {"multi_match": {"query": query, "fields": fields}}}
    return search_manager.execute_query(list_of_indices, query_body, limit, with_grouping)


@additional_router.get("/document/op/search")
async def search_query_string(index_list: IndexList, limit: Union[int, None] = DEFAULT_LIMIT,
                              field: Union[str, None] = "", query: Union[str, None] = "",
                              with_grouping: Union[bool, None] = True):
    """Search by using operators with query string and size parameter"""
    list_of_indices = validate_indices(index_list.indices)
    logger.info("Searching for " + query + " in the field " + field + " in indices: " + str(list_of_indices))
    query_body = {"query": {"query_string": {"query": query, "default_field": field}}}
    return search_manager.execute_query(list_of_indices, query_body, limit, with_grouping)


@additional_router.get("/document/op/fuzzySearch")
async def search_fuzzy(index_list: IndexList, limit: Union[int, None] = DEFAULT_LIMIT,
                       field: Union[str, None] = "", query: Union[str, None] = "", fuzziness: Union[int, None] = 2,
                       with_grouping: Union[bool, None] = True):
    """Search by using fuzzy with query string and size parameter"""
    list_of_indices = validate_indices(index_list.indices)
    logger.info("Searching for " + query + " in the field " + field + " in indices: " + str(list_of_indices))
    query_body = {"query": {"fuzzy": {field: {"value": query, "fuzziness": fuzziness}}}}
    return search_manager.execute_query(list_of_indices, query_body, limit, with_grouping)


@additional_router.get("/document/op/searchByRange")
async def search_range(index_list: IndexList, limit: Union[int, None] = DEFAULT_LIMIT, field: Union[str, None] = "",
                       query: Union[str, None] = "", lte: Union[int, None] = 0, gte: Union[int, None] = 100,
                       with_grouping: Union[bool, None] = True):
    """Search by specifying a range of values for a field"""
    list_of_indices = validate_indices(index_list.indices)
    logger.info("Searching for " + query + " in the field " + field + " in indices: " + str(list_of_indices))
    query_body = {"query": {"range": {field: {"gte": gte, "lte": lte}}}}
    return search_manager.execute_query(list_of_indices, query_body, limit, with_grouping)


@additional_router.post("/document/op/searchByQuery")
async def search_by_query(index_list: IndexList, limit: Union[int, None] = DEFAULT_LIMIT,
                          query_body: Union[str, None] = "", with_grouping: Union[bool, None] = True):
    list_of_indices = validate_indices(index_list.indices)
    logger.info("Searching for " + query_body + " in index(/indices): " + str(list_of_indices))
    return search_manager.execute_query(list_of_indices, query_body, limit, with_grouping)



import logging
import time
from typing import Callable

from fastapi import Request, Response
from fastapi.routing import APIRoute
from starlette.background import BackgroundTask
from starlette.responses import StreamingResponse

logging.basicConfig(filename='app/logs/search.log', level=logging.DEBUG,
                    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

# formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
#
# file_handler = logging.FileHandler('logs/search.log')
# file_handler.setFormatter(formatter)


def log_info(req_body, res_body):
    logging.info("Request Body: " + str(req_body))
    logging.info("Response Body: " + str(res_body))


class LoggingRoute(APIRoute):
    def get_route_handler(self) -> Callable:
        original_route_handler = super().get_route_handler()

        async def custom_route_handler(request: Request) -> Response:
            before = time.time()
            req_body = await request.body()
            response = await original_route_handler(request)
            duration = time.time() - before
            response.headers["X-Response-Time"] = str(duration)

            if isinstance(response, StreamingResponse):
                res_body = b''
                async for item in response.body_iterator:
                    res_body += item

                task = BackgroundTask(log_info, req_body, res_body)
                return Response(content=res_body, status_code=response.status_code,
                                headers=dict(response.headers), media_type=response.media_type, background=task)
            else:
                res_body = response.body
                response.background = BackgroundTask(log_info, req_body, res_body)
                return response

        return custom_route_handler



import logging
import os
import socket

from dotenv import load_dotenv

from app.searchManager import SearchManager

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
load_dotenv()
SERVICE_URI = os.getenv("SERVICE_URI")
if SERVICE_URI is None:
    logger.warning(
        "Update SERVICE_URI to your cluster uri. Current value for SERVICE_URI={"
        "SERVICE_URI}")
    logger.info(
        "As a workaround, Going to put your hostname as SERVICE_URI")

    SERVICE_URI = socket.gethostbyname(socket.gethostname())

logger.info("SERVICE_URI={}".format(SERVICE_URI))


SERVICE_PORT = int(os.getenv("SERVICE_PORT"))
if SERVICE_PORT is None:
    logger.warning(
        "Update SERVICE_PORT to your cluster uri. Current value for SERVICE_PORT={"
        "SERVICE_PORT}")
    logger.info(
        "As a workaround, Going to put 9000 as SERVICE_PORT")

    SERVICE_PORT = 9000

logger.info("SERVICE_PORT={}".format(SERVICE_PORT))

search_manager = SearchManager()



import logging
import os

from dotenv import load_dotenv


logger = logging.getLogger(__name__)
DEFAULT_LIMIT = 20

load_dotenv()
ELASTIC_SEARCH_SERVICE_URI = os.getenv("ELASTIC_SEARCH_SERVICE_URI")
if ELASTIC_SEARCH_SERVICE_URI is None:
    logger.error(
        "Update ELASTIC_SEARCH_SERVICE_URI to your cluster uri. Current value for ELASTIC_SEARCH_SERVICE_URI={"
        "ELASTIC_SEARCH_SERVICE_URI}")
    exit(-1)

# Password for the 'elastic' user generated by Elasticsearch
ELASTIC_PASSWORD = os.getenv("ELASTIC_PASSWORD")
if ELASTIC_PASSWORD is None:
    logger.error(
        "Update ELASTIC_PASSWORD to your cluster uri. Current value for ELASTIC_PASSWORD={ELASTIC_PASSWORD}")
    exit(-1)


logger.info("Elastic Environment Variables received Successfully!")



from enum import Enum

from pydantic import BaseModel, Field


class SearchIndex(str, Enum):
    FOLDER = "library.folder"
    COURSE = "library.course"
    MAJOR = "library.major"
    FACULTY = "library.faculty"
    USER = "library.user"


class GENDER(str, Enum):
    MALE = "male"
    FEMALE = "female"
    CLUB = "club"
    UNKNOWN = "unknown"


class IndexList(BaseModel):
    indices: list = Field(examples=[list(SearchIndex)])


class DetailedCriteria(BaseModel):
    index_list: list = Field(examples=[list(SearchIndex)])
    exact_match: dict
    query: dict = Field(examples=[{"*": "تلخيص سجود حمايل فيزياء"}])


class Criteria(BaseModel):
    index_list: list = Field(examples=[list(SearchIndex)])
    query: str = Field(examples=["تلخيص سجود حمايل فيزياء"])


class SearchInfo(BaseModel):
    id: str = Field(examples=["123e4567-e89b-12d3-a456-426614174000"])
    type: SearchIndex = Field(examples=[SearchIndex.FOLDER])


class Folder(SearchInfo):
    name: str = Field(examples=["new folder name"])
    description: str = Field(examples=["This is a description for the new folder"])
    courseNameArabic: str = Field(examples=["إسم المجلد بالعربية"])
    courseNameEnglish: str = Field(examples=["new folder courseNameEnglish"])


class Course(SearchInfo):
    nameEnglish: str = Field(examples=["new course nameEnglish"])
    nameArabic: str = Field(examples=["إسم المساق بالعربية"])
    symbol: str = Field(examples=["COURSE123"])


class Major(SearchInfo):
    nameEnglish: str = Field(examples=["new major nameEnglish"])
    nameArabic: str = Field(examples=["إسم التخصص بالعربية"])
    symbol: str = Field(examples=["MAJOR234"])
    description: str = Field(examples=["This is a description for the new major"])


class Faculty(SearchInfo):
    nameEnglish: str = Field(examples=["new faculty nameEnglish"])
    nameArabic: str = Field(examples=["إسم الكلية بالعربية"])
    symbol: str = Field(examples=["FACULTY345"])
    description: str = Field(examples=["This is a description for the new faculty"])


class User(SearchInfo):
    userName: str = Field(examples=["1201201"])
    name: str = Field(examples=["Ahmad Mohamad"])
    gender: GENDER = Field(examples=[GENDER.MALE])
    majorArabicName: str = Field(examples=["هندسة الحاسوب"])
    majorEnglishName: str = Field(examples=["Computer Engineering"])
